
#|
Copyright (c) 2011 Christopher K. Riesbeck

Permission is hereby granted, free of charge, to any person obtaining 
a copy of this software and associated documentation files (the "Software"), 
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom the 
Software is furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included 
in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR 
OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
OTHER DEALINGS IN THE SOFTWARE.
|#

;;; Change Log
;;;
;;; 11/25/2011: added webgen/ to *WEBGEN-SOURCE-DIR* [CKR]

(defpackage #:webgen
  (:use #:common-lisp #:net.aserve #:cs325)
  (:export #:make))

(in-package #:webgen)

;;; where new webapp directories are made
(defparameter *webgen-target-dir* *cs325-home*)

;;; where the webapp template files come from
(defparameter *webgen-source-dir* 
  (merge-pathnames "webgen/webgen-templates/" *webgen-target-dir*))

(defun make (name)
  (let* ((dirname (if (symbolp name) (string-downcase name) name))
         (dirpath (merge-pathnames (make-pathname :directory (list :relative dirname))
                                   *webgen-target-dir*)))
    (when (or (null (probe-file dirpath))
              (y-or-n-p "~A directory already exists -- replace?" dirpath))
      (ensure-directories-exist dirpath)
      (dolist (source (directory *webgen-source-dir*))
        (copy-template source (merge-pathnames dirpath source) dirname))
      dirpath)))

(defun copy-template (source dest dirname)
  (with-open-file (out dest :direction :output :if-exists :new-version)
    (with-open-file (in source)
      (do ((line (read-line in nil) (read-line in nil)))
          ((null line))
        (write-line (string-replace "webapp" dirname line) out)))))

(defun string-replace (old new str)
  (let ((len (length old)))
    (with-output-to-string (out)
      (do ((pos (search old str :start2 0)
                (search old str :start2 (+ pos len)))
           (start 0 (+ pos len)))
          ((null pos) (write-string str out :start start))
        (write-string str out :start start :end pos)
        (write-string new out)))))
      

